package com.ruoyi.jjh.declaration.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.jjh.declaration.entity.BmsBigStrongAward;
import com.ruoyi.jjh.declaration.entity.BmsDeclarationRecords;
import com.ruoyi.jjh.declaration.entity.BmsFundingDetail;
import com.ruoyi.jjh.declaration.entity.dto.BmsBigStrongAwardAddDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsBigStrongAwardUpdateDto;
import com.ruoyi.jjh.declaration.entity.dto.BmsFundingDetailAddDto;
import com.ruoyi.jjh.declaration.entity.vo.BmsBigStrongAwardQueryVo;
import com.ruoyi.jjh.declaration.mapper.BmsBigStrongAwardMapper;
import com.ruoyi.jjh.declaration.service.*;
import com.ruoyi.jjh.ent.entity.JProject;
import com.ruoyi.jjh.ent.service.JProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * 做大做强奖详情Service业务层处理
 *
 * @author farben
 * @date 2023-08-25
 */
@Service
@Lazy
public class BmsBigStrongAwardServiceImpl extends ServiceImpl<BmsBigStrongAwardMapper, BmsBigStrongAward> implements IBmsBigStrongAwardService {

    @Autowired
    private ICommonService iCommonService;

    @Autowired
    private IBmsFundingDetailService iBmsFundingDetailService;

    @Autowired
    private IBmsDeclarationRecordsService iBmsDeclarationRecordsService;

    @Resource
    private IBmsTemplateInfoService iBmsTemplateInfoService;

    @Resource
    private JProjectService jProjectService;
    /**
     * 查询做大做强奖详情
     *
     * @param id 做大做强奖详情主键
     * @return 做大做强奖详情
     */
    @Override
    public BmsBigStrongAwardQueryVo selectBmsBigStrongAwardById(Long id) {
        return baseMapper.selectBmsBigStrongAwardById(id);
    }

    /**
     * 查询做大做强奖详情列表
     *
     * @param bmsBigStrongAward 做大做强奖详情
     * @return 做大做强奖详情
     */
    @Override
    public List<BmsBigStrongAward> selectBmsBigStrongAwardList(BmsBigStrongAward bmsBigStrongAward) {
        return baseMapper.selectBmsBigStrongAwardList(bmsBigStrongAward);
    }

    /**
     * 新增做大做强奖详情
     *
     * @param bmsBigStrongAwardAddDto 做大做强奖详情
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertBmsBigStrongAward(BmsBigStrongAwardAddDto bmsBigStrongAwardAddDto) {

        // check 企业信息
        Long enterpriseId = iCommonService.checkEnterprise(bmsBigStrongAwardAddDto.getTyshxydm(), bmsBigStrongAwardAddDto.getTemplateRecordId());
        // check 申报信息
        iCommonService.checkDeclaration(bmsBigStrongAwardAddDto.getTyshxydm()
                , bmsBigStrongAwardAddDto.getTemplateRecordId());
        if (enterpriseId == 0) {
            return 0;
        }
        BmsBigStrongAward bmsBigStrongAward = new BmsBigStrongAward();
        BeanUtil.copyProperties(bmsBigStrongAwardAddDto, bmsBigStrongAward);
        bmsBigStrongAward.setEnterpriseId(enterpriseId);
        // 新增做大做强奖详情
        int num = baseMapper.insert(bmsBigStrongAward);

        List<BmsFundingDetailAddDto> fundingDetailList = bmsBigStrongAwardAddDto.getFundingDetailList();
        List<BmsFundingDetail> fundingInfoList = new ArrayList<>();
        fundingDetailList.forEach(x -> {
            BmsFundingDetail fundingInfo = new BmsFundingDetail();
            BeanUtil.copyProperties(x, fundingInfo);
            fundingInfo.setType(2);
            fundingInfo.setTemplateRecordId(bmsBigStrongAwardAddDto.getTemplateRecordId());
            fundingInfo.setDetailId(bmsBigStrongAward.getId());
            fundingInfoList.add(fundingInfo);
        });
        // 批量新增资金信息
        iBmsFundingDetailService.saveOrUpdateBatch(fundingInfoList);
        // 修改在线申报记录
        BmsDeclarationRecords declarationRecords = new BmsDeclarationRecords();
        declarationRecords.setMatter("提交做大做强奖补申请");
        declarationRecords.setDetailId(bmsBigStrongAward.getId());
        declarationRecords.setStatus(1L);
        declarationRecords.setId(bmsBigStrongAwardAddDto.getDeclarationId());
        iBmsDeclarationRecordsService.updateById(declarationRecords);


        //获取某个在线记录id可以添加到项目库的数据
        JProject addProject = iBmsDeclarationRecordsService.getAddProject(bmsBigStrongAwardAddDto.getDeclarationId());
        addProject.setStatus(1);
        String jsonString = jProjectService.getJsonString(fundingInfoList);
        JSONObject mergedObj = new JSONObject();
        mergedObj.putAll(JSONObject.parseObject(jsonString));
        if (bmsBigStrongAward.getIsGainRewards() == 0) {
            bmsBigStrongAward.setNewRewardDescription("是");
        } else {
            bmsBigStrongAward.setNewRewardDescription("否");
        }
        mergedObj.putAll(JSONObject.parseObject(JSONObject.toJSONString(bmsBigStrongAward)));
        addProject.setOtherJson(mergedObj.toJSONString());
        addProject.setFileJson("{" +
                '"' + "企业近3年发展情况及未来发展计划" + '"' + ":" + '"' + bmsBigStrongAward.getDevelopmentPlan() + '"' +
                "}");
        jProjectService.save(addProject);
        declarationRecords.setJjhProjectId(addProject.getId());
        iBmsDeclarationRecordsService.updateById(declarationRecords);
        // 申请审批
        iCommonService.insertAuditRecord(declarationRecords.getId(), 0);

        return num;
    }


    /**
     * 修改做大做强奖详情
     *
     * @param bmsBigStrongAwardUpdateDto 做大做强奖详情
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateBmsBigStrongAward(BmsBigStrongAwardUpdateDto bmsBigStrongAwardUpdateDto) {
        int num = 0;
        try {
            BmsDeclarationRecords newRecord = new BmsDeclarationRecords();
            newRecord.setId(bmsBigStrongAwardUpdateDto.getDeclarationRecordsId());
            newRecord.setStatus(bmsBigStrongAwardUpdateDto.getStatus());
            iBmsDeclarationRecordsService.updateById(newRecord);

            num = baseMapper.updateBmsBigStrongAward(bmsBigStrongAwardUpdateDto);
            List<BmsFundingDetail> fundingDetailList = bmsBigStrongAwardUpdateDto.getFundingDetailList();
            iBmsFundingDetailService.saveOrUpdateBatch(fundingDetailList);
            // 申请审批
            if (bmsBigStrongAwardUpdateDto.getStatus() == 1) {
                JProject jP = new JProject();
                BmsBigStrongAward bmsBigStrongAward = new BmsBigStrongAward();
                BeanUtil.copyProperties(bmsBigStrongAwardUpdateDto, bmsBigStrongAward);
                jP.setId(iBmsDeclarationRecordsService.getById(bmsBigStrongAwardUpdateDto.getDeclarationRecordsId()).getJjhProjectId());
                jP.setStatus(1);
                JSONObject mergedObj = new JSONObject();
                mergedObj.putAll(JSONObject.parseObject(jProjectService.getJsonString(fundingDetailList)));
                mergedObj.putAll(JSONObject.parseObject(JSONObject.toJSONString(bmsBigStrongAward)));
                jP.setOtherJson(mergedObj.toJSONString());
                jP.setFileJson("{" +
                        '"' + "企业近3年发展情况及未来发展计划" + '"' + ":" + '"' + bmsBigStrongAward.getDevelopmentPlan() + '"' +
                        "}");
                //修改项目
                jProjectService.updateById(jP);
                // 修改审批状态
                iCommonService.updateAuditRecord(bmsBigStrongAwardUpdateDto.getDeclarationRecordsId(), 0);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return num;
    }

    /**
     * 批量删除做大做强奖详情
     *
     * @param ids 需要删除的做大做强奖详情主键
     * @return 结果
     */
    @Override
    public int deleteBmsBigStrongAwardByIds(Long[] ids) {
        return baseMapper.deleteBmsBigStrongAwardByIds(ids);
    }

    /**
     * 删除做大做强奖详情信息
     *
     * @param id 做大做强奖详情主键
     * @return 结果
     */
    @Override
    public int deleteBmsBigStrongAwardById(Long id) {
        return baseMapper.deleteBmsBigStrongAwardById(id);
    }
}
