package com.ruoyi.jjh.declaration.single.service.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.json.JSONUtil;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.exception.user.UserPasswordNotMatchException;
import com.ruoyi.common.utils.MessageUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.framework.manager.AsyncManager;
import com.ruoyi.framework.manager.factory.AsyncFactory;
import com.ruoyi.framework.security.context.AuthenticationContextHolder;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.jjh.declaration.single.service.SingleLoginService;
import com.ruoyi.jjh.declaration.single.util.HMAC256Config;
import com.ruoyi.jjh.ent.service.SmsAlertsService;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

/**
 * @author du
 * @since 2024/6/4 15:17
 */
@Service
public class SingleLoginServiceImpl implements SingleLoginService {

    @Resource
    private SmsAlertsService smsAlertsService;

    @Resource
    private AuthenticationManager authenticationManager;

    @Resource
    private TokenService tokenService;

    @Resource
    private ISysUserService userService;

    @Value("${isTiming}")
    private Boolean isTiming;

    @Value("${enterpriseAppId}")
    private String enterpriseAppId;

    /**
     * 保存用户信息 换取本系统token
     */
    @Override
    public String singleLogin(String userName, String nickName, Long id, String userType,String phone) {
        // 用户验证
        Authentication authentication;
        try {
            SysUser user = new SysUser();
            user.setUserName(userName);
            user.setNickName(nickName);
            user.setPassword("admin123");
            user.setPhonenumber(phone);
            Long[] roles = new Long[1];
            if("01".equals(userType)){
                roles[0]=101L;
            }else if("02".equals(userType)){
                //在这里可以判断具体要给什么权限  要给dept部门id
                roles[0]=102L;
                user.setDeptId(100L);
                user.setZwuser(userName);
            }
            user.setRoleIds(roles);
            user.setStatus("0");
            user.setUserType(userType);
            user.setEnterpriseId(id);
            //如果用户名也就是信用代码没有重复的就新增用户
            if (userService.checkUserNameUnique(user))
            {
                user.setPassword(SecurityUtils.encryptPassword(user.getPassword()));
                userService.insertUser(user);
                if("02".equals(userType)&isTiming){
                    if(user.getPhonenumber()!=null){
                        smsAlertsService.addSsmPhone(user.getPhonenumber());
                    }
                }
            }
            //新增user账号密码
            UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(userName, "admin123");
            AuthenticationContextHolder.setContext(authenticationToken);
//             该方法会去调用UserDetailsServiceImpl.loadUserByUsername
            authentication = authenticationManager.authenticate(authenticationToken);
        } catch (Exception e) {
            if (e instanceof BadCredentialsException) {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(userName, Constants.LOGIN_FAIL, MessageUtils.message("user.password.not.match")));
                throw new UserPasswordNotMatchException();
            } else {
                AsyncManager.me().execute(AsyncFactory.recordLogininfor(userName, Constants.LOGIN_FAIL, e.getMessage()));
                throw new ServiceException(e.getMessage());
            }
        } finally {
            AuthenticationContextHolder.clearContext();
        }
        AsyncManager.me().execute(AsyncFactory.recordLogininfor(userName, Constants.LOGIN_SUCCESS, MessageUtils.message("user.login.success")));
        LoginUser loginUser = (LoginUser) authentication.getPrincipal();
        // 生成token
        Map<String, Object> token = tokenService.createToken(loginUser);
        return String.valueOf(token.get("access_token"));
    }


    /**
     * 正式环境需要的请求头
     * @return
     */
    @Override
    public Map<String, String> generationHeader() {
        // 定义时间格式化器
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmss");
        String format = LocalDateTime.now().format(formatter);
        //正式端需要请求头
        HashMap<String, String> headers = new HashMap<>();//存放请求头，可以存放多个请求头
        headers.put("appid", enterpriseAppId);
        headers.put("time", format);
        headers.put("sign", DigestUtil.md5Hex(enterpriseAppId+"03f71c58-39e9-11ef-bbd6-fa163e2b2e10"+format));
        return headers;
    }

}

